@@ -30,6 +30,25 @@ OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 #include "sieve_interface.h"
 
+enum enum_value {
+
+  IMAP_ENUM_ZERO = 0,
+
+  IMAP_ENUM_SIEVE_EXTENSIONS_COPY = (1<<11),
+  IMAP_ENUM_SIEVE_EXTENSIONS_SUBADDRESS = (1<<10),
+  IMAP_ENUM_SIEVE_EXTENSIONS_REGEX = (1<<9),
+  IMAP_ENUM_SIEVE_EXTENSIONS_RELATIONAL = (1<<8),
+  IMAP_ENUM_SIEVE_EXTENSIONS_BODY = (1<<7),
+  IMAP_ENUM_SIEVE_EXTENSIONS_ENVELOPE = (1<<6),
+  IMAP_ENUM_SIEVE_EXTENSIONS_INCLUDE = (1<<5),
+  IMAP_ENUM_SIEVE_EXTENSIONS_NOTIFY = (1<<4),
+  IMAP_ENUM_SIEVE_EXTENSIONS_IMAPFLAGS = (1<<3),
+  IMAP_ENUM_SIEVE_EXTENSIONS_VACATION = (1<<2),
+  IMAP_ENUM_SIEVE_EXTENSIONS_REJECT = (1<<1),
+  IMAP_ENUM_SIEVE_EXTENSIONS_FILEINTO = (1<<0)
+};
+#define EXTENSIONS_ALL ((1 << 12)-1)
+
 struct sieve_interp {
     /* standard callbacks for actions */
     sieve_callback *redirect, *discard, *reject, *fileinto, *keep;
@@ -39,6 +58,8 @@ struct sieve_interp {
     sieve_get_size *getsize;
     sieve_get_header *getheader;
     sieve_get_envelope *getenvelope;
+    sieve_get_body *getbody;
+    sieve_get_include *getinclude;
 
     sieve_parse_error *err;
 
