@@ -69,38 +69,49 @@
      SIZE = 285,
      ADDRESS = 286,
      ENVELOPE = 287,
-     COMPARATOR = 288,
-     IS = 289,
-     CONTAINS = 290,
-     MATCHES = 291,
-     REGEX = 292,
-     COUNT = 293,
-     VALUE = 294,
-     OVER = 295,
-     UNDER = 296,
-     GT = 297,
-     GE = 298,
-     LT = 299,
-     LE = 300,
-     EQ = 301,
-     NE = 302,
-     ALL = 303,
-     LOCALPART = 304,
-     DOMAIN = 305,
-     USER = 306,
-     DETAIL = 307,
-     DAYS = 308,
-     ADDRESSES = 309,
-     SUBJECT = 310,
-     MIME = 311,
-     METHOD = 312,
-     ID = 313,
-     OPTIONS = 314,
-     LOW = 315,
-     NORMAL = 316,
-     HIGH = 317,
-     ANY = 318,
-     MESSAGE = 319
+     BODY = 288,
+     COMPARATOR = 289,
+     IS = 290,
+     CONTAINS = 291,
+     MATCHES = 292,
+     REGEX = 293,
+     COUNT = 294,
+     VALUE = 295,
+     OVER = 296,
+     UNDER = 297,
+     GT = 298,
+     GE = 299,
+     LT = 300,
+     LE = 301,
+     EQ = 302,
+     NE = 303,
+     ALL = 304,
+     LOCALPART = 305,
+     DOMAIN = 306,
+     USER = 307,
+     DETAIL = 308,
+     RAW = 309,
+     TEXT = 310,
+     CONTENT = 311,
+     DAYS = 312,
+     ADDRESSES = 313,
+     SUBJECT = 314,
+     FROM = 315,
+     HANDLE = 316,
+     MIME = 317,
+     METHOD = 318,
+     ID = 319,
+     OPTIONS = 320,
+     LOW = 321,
+     NORMAL = 322,
+     HIGH = 323,
+     ANY = 324,
+     MESSAGE = 325,
+     INCLUDE = 326,
+     PERSONAL = 327,
+     GLOBAL = 328,
+     RETURN = 329,
+     COPY = 330
    };
 #endif
 /* Tokens.  */
@@ -134,45 +145,56 @@
 #define SIZE 285
 #define ADDRESS 286
 #define ENVELOPE 287
-#define COMPARATOR 288
-#define IS 289
-#define CONTAINS 290
-#define MATCHES 291
-#define REGEX 292
-#define COUNT 293
-#define VALUE 294
-#define OVER 295
-#define UNDER 296
-#define GT 297
-#define GE 298
-#define LT 299
-#define LE 300
-#define EQ 301
-#define NE 302
-#define ALL 303
-#define LOCALPART 304
-#define DOMAIN 305
-#define USER 306
-#define DETAIL 307
-#define DAYS 308
-#define ADDRESSES 309
-#define SUBJECT 310
-#define MIME 311
-#define METHOD 312
-#define ID 313
-#define OPTIONS 314
-#define LOW 315
-#define NORMAL 316
-#define HIGH 317
-#define ANY 318
-#define MESSAGE 319
+#define BODY 288
+#define COMPARATOR 289
+#define IS 290
+#define CONTAINS 291
+#define MATCHES 292
+#define REGEX 293
+#define COUNT 294
+#define VALUE 295
+#define OVER 296
+#define UNDER 297
+#define GT 298
+#define GE 299
+#define LT 300
+#define LE 301
+#define EQ 302
+#define NE 303
+#define ALL 304
+#define LOCALPART 305
+#define DOMAIN 306
+#define USER 307
+#define DETAIL 308
+#define RAW 309
+#define TEXT 310
+#define CONTENT 311
+#define DAYS 312
+#define ADDRESSES 313
+#define SUBJECT 314
+#define FROM 315
+#define HANDLE 316
+#define MIME 317
+#define METHOD 318
+#define ID 319
+#define OPTIONS 320
+#define LOW 321
+#define NORMAL 322
+#define HIGH 323
+#define ANY 324
+#define MESSAGE 325
+#define INCLUDE 326
+#define PERSONAL 327
+#define GLOBAL 328
+#define RETURN 329
+#define COPY 330
 
 
 
 
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 131 "sieve.y"
+#line 148 "sieve.y"
 {
     int nval;
     char *sval;
@@ -183,11 +205,12 @@ typedef union YYSTYPE
     struct vtags *vtag;
     struct aetags *aetag;
     struct htags *htag;
+    struct btags *btag;
     struct ntags *ntag;
     struct dtags *dtag;
 }
 /* Line 1489 of yacc.c.  */
-#line 191 "sieve.h"
+#line 214 "sieve.h"
 	YYSTYPE;
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
