@@ -85,6 +85,8 @@ static const char *unfold_header(const char *str)
 		if (str[i] == '\n') {
 			new_str[j++] = ' ';
 			i++;
+			if (str[i] == '\0')
+				break;
 			i_assert(str[i] == ' ' || str[i] == '\t');
 		} else {
 			new_str[j++] = str[i];
@@ -540,7 +542,6 @@ static int sieve_reject(void *ac,
 	*errmsg = "Error sending mail";
 	return SIEVE_FAIL;
     }
-    return SIEVE_FAIL;
 }
 
 static void get_flags(const sieve_imapflags_t *sieve_flags,
@@ -748,7 +749,7 @@ static int send_response(void *ac,
     fprintf(f, "Date: %s\r\n", message_date_create(ioloop_time));
     
     fprintf(f, "X-Sieve: %s\r\n", SIEVE_VERSION);
-    fprintf(f, "From: <%s>\r\n", src->fromaddr);
+    fprintf(f, "From: %s\r\n", src->fromaddr);
     fprintf(f, "To: <%s>\r\n", src->addr);
     fprintf(f, "Subject: %s\r\n", str_sanitize(src->subj, 80));
     if (md->id) fprintf(f, "In-Reply-To: %s\r\n", md->id);
@@ -898,14 +899,14 @@ static int
 dovecot_sieve_compile(script_data_t *sdata, const char *script_path,
 		      const char *compiled_path)
 {
-	struct stat st, st2;
+	struct stat st, st2, stlnk;
 	sieve_script_t *script;
 	bytecode_info_t *bc;
 	const char *temp_path;
 	FILE *f;
 	int fd, ret;
 
-	if (stat(script_path, &st) < 0) {
+	if (lstat(script_path, &st) < 0) {
 		if (errno == ENOENT) {
 			if (getenv("DEBUG") != NULL) {
 				i_info("cmusieve: Script not found: %s",
@@ -913,20 +914,28 @@ dovecot_sieve_compile(script_data_t *sdata, const char *script_path,
 			}
 			return 0;
 		}
-		i_error("stat(%s) failed: %m", script_path);
+		i_error("lstat(%s) failed: %m", script_path);
 		return -1;
 	}
 	if (S_ISDIR(st.st_mode)) {
 		i_error("%s should be a file, not a directory", script_path);
 		return -1;
 	}
+	/* in case of symbolic link compare linked files date, too */
+	if (S_ISLNK(st.st_mode)) {
+		if (stat(script_path, &stlnk) < 0) {
+			i_error("stat(%s) failed: %m", script_path);
+			return -1;
+		}
+	}
 	if (stat(compiled_path, &st2) < 0) {
 		if (errno != ENOENT) {
 			i_error("stat(%s) failed: %m", script_path);
 			return -1;
 		}
 	} else {
-		if (st.st_mtime <= st2.st_mtime)
+		if (st.st_mtime <= st2.st_mtime && 
+				(!S_ISLNK(st.st_mode) || stlnk.st_mtime <= st2.st_mtime))
 			return 1;
 	}
 
